# wet/service/rss.py
#
#

""" rss module. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from wet.utils.url import get_url, strip_html, need_redirect
from wet.utils.time import short_date
from wet.errors import error
from wet.clock import Repeater
from wet.kernel import kernel
from wet.thing import Thing

import time

## DEFINES

key_list = ["published", "title", "link"]

seen = Thing()
seen.list = []
seen.results = "rss"
seen.prefix = "results"

## display function

def display(*args, **kwargs):
    entry = args[0]
    result = ""
    for key in key_list:
        if entry.get(key, None): result += "%s - " % entry[key]
    return result[:-3]

## get function

def get(url):
    import wet.extra.feedparser as fp
    result = []
    try:
        data = get_url("GET", url)
        url = need_redirect(data)
        if url: data = get_url("GET", url) 
        data = fp.parse(data.read())
    except Exception as ex: error() ; return
    for entry in data["entries"]:
        o = Thing(entry)
        o.service = "rss"
        s = o.slice(key_list)
        if "summary" in o: o["summary"] = strip_html(o["summary"]).rstrip()
        o.prefix = "rss"
        o.short = short_date(time.ctime())
        yield o

## look function

def rss_fetch(*args, **kwargs):
    global seen
    for fn in kernel.all():
        obj = Thing().load(fn)
        if "deleted" in obj and obj.deleted: continue
        if "rss" not in obj: continue
        if "http" not in obj.rss: continue
        for o in get(obj.rss):
            if o.link in seen.list: continue
            seen.list.append(o.link)
            o.sync()
            kernel.announce(display(o))
            if "summary" in o: kernel.announce(o.summary)
    seen.sync()

kernel.register("rss.fetch", rss_fetch)

## RSS

class RSS(Repeater): pass

##

def write_rss(event):
    o = Thing(event)
    o.prefix = "rss"  
    o.rss = event.parsed.rest
    o.service = "rss"
    path = o.save()  
    event.ok()

kernel.register("write.rss", write_rss)


## INIT

def init(*args, **kwargs):
    global seen
    last_rss = kernel.last("results", "rss")
    if last_rss: seen = last_rss
    rss = RSS(rss_fetch, 600)
    kernel.put(rss.start)

